/* This program receives an tm8e tape image from the serial port from the
   PDP8 dump program.  It dumps the files from the tape in individual files.
   It needs a config file dumprest.cfg or
   $HOME/.dumprest.cfg with the format defined in config.c

   This program should be running before the PDP8 end is started.

   On the PC ctrl-break will terminate the program
*/
#ifdef PC
#include "encom.h"
#else
#include <termios.h>
#include <unistd.h>
#include <memory.h>
#endif

#include <stdio.h>
#include <fcntl.h>
#include <time.h>
#include <signal.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>

#define ARRAYSIZE(a) (sizeof(a) / sizeof(a[0]))

int terminate = 0;

#include "config.c"
#include "comm.c"
unsigned int get_word(int fd) {
   unsigned char buf[2];
   unsigned int word;
   int c;
   int cnt = 0;

      //c = ser_read(fd,(char *)buf,sizeof(buf));
   do {
      c = ser_read(fd,(char *)&buf[cnt],2-cnt);
      if (c < 0) {
         perror("Serial read failed");
         exit(1);
      }
      cnt += c;
      if (terminate) {
         exit(1);
      }
   } while (cnt < 2); 
   word = ((buf[1] & 077) << 6) + (buf[0] & 077);

   return word;
}

int main(argc,argv)
   int argc;
   char *argv[];
{
   int fd,i;
   FILE *out;
   char filename[256];
   char serial_dev[256];
   long baud;
   int two_stop;
   unsigned int chksum = 0;
   uint16_t word;
   unsigned int msr, fsr, blklen;
   int last_fm = 0;
   int filenum = 1;
   int records = 0;
   unsigned char buf[256];
   int c;
   int err;

   setup_config(&baud,&two_stop,serial_dev);

#if 0
   if (argc > 1) {
      strcpy(filename,argv[1]);
   } else {
      printf("Enter file name to receive\n");
      fflush(stdout);
      scanf("%s",filename);
   }
#endif

   sprintf(filename, "file%04d",filenum);
#ifdef PC
   out = fopen(filename,"wb");
#else
   out = fopen(filename,"w");
#endif
   if (out < 0) {
      fprintf(stderr,"On file %s ",filename);
      perror("open failed");
      exit(1);
   }

#if 0
      /* For testing read from file, only works in unix version */
   fd = open("dat",O_RDONLY,0666);
   if (fd < 0) {
      perror("Open failed on dat");
      exit(1);
   }
#else
   fd = init_comm(serial_dev,baud,two_stop);
   do {
      c = ser_read(fd,(char *)buf,sizeof(buf));
   } while (c != 0);
#endif
   while(!terminate) {
      word = get_word(fd);
      if (word == 05000) {
         printf("Done\n");
         exit(0);
      }
      if (word != 04000) {
         printf("Bad sync word %04o\n", word);
         exit(0);
      }
      blklen = get_word(fd);
      msr = get_word(fd);
      fsr = get_word(fd);
      if (blklen == 0) { 
         printf("MSR %04o FSR %04o\n", msr, fsr);
      }
      err = 0;
      if (msr & 00200) {
         printf("parity ");
         err = 1;
      }
      if (msr & 00100) {
         printf("file mark ");
         err = 1;
      }
      if (msr & 00020) {
         printf("data late ");
         err = 1;
      }
      if (msr & 00010) {
         printf("end of tape ");
         err = 1;
      }
      if (msr & 00001) {
         printf("illegal function ");
         err = 1;
      }
      if (fsr & 00004) {
         printf("vertical parity ");
         err = 1;
      }
      if (fsr & 00002) {
         printf("CRC ");
         err = 1;
      }
      if (fsr & 00001) {
         printf("Longitudinal parity ");
         err = 1;
      }
      if (err) {
         printf("\n");
      }
      chksum = 0;
      for (i = 0; i < blklen; i++) {
         word = get_word(fd);
         chksum = chksum + word;
         fwrite(&word,2,1,out);
      }
      word = get_word(fd);
      chksum = chksum & 07777;
      if (word != chksum) {
         printf("Checksum mismatch %04o %04o record len %d\n", chksum, word, blklen);
      }
      records++;

      // File mark
      if (msr & 00100) {
         if (last_fm) {
            printf("End of volume\n");
         } else {
            printf("Filemark, records %d len %d\n", records, blklen);
            records = 0;
            fclose(out);
            sprintf(filename, "file%04d",++filenum);
            out = fopen(filename,"w");
         }
         last_fm = 1;
      } else {
         last_fm = 0;
      }
   }
   return 1;
}
