#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <netinet/tcp.h>

#include <termios.h>
#include <unistd.h>
#include <memory.h>
#include <signal.h>

int terminate = 0;

#define SER 1

#define PORT 5000

#define ARRAYSIZE(a) (sizeof(a) / sizeof(a[0]))

#include "comm.c"
#include "config.c"



int client_fd;

uint16_t get_word() {
   char bytes[2];
   int rc =read(client_fd, bytes, 2);
   if (rc == 1) {
      rc =read(client_fd, &bytes[1], 1);
      if (rc == 1) {
         rc++;
      }
   }
   if (rc != 2) {
      printf("Bad socket read %d\n", rc);
      close(client_fd);
      exit(1);
   }
   return (((bytes[0] << 6) & 07700) | (bytes[1] & 077));
}

void send_word(uint16_t word) {
   char bytes[2];
   bytes[0] = (word >> 6) & 077;
   bytes[1] = word & 077;
   write(client_fd, bytes, 2);
}

int main(int argc, char *argv[]) {
   int fd;

    int status;
    struct sockaddr_in serv_addr;
    int option = 1;

   char serial_dev[256];
   long baud;
   int two_stop;
   int file = 0, records = 0;

#if SER
   setup_config(&baud,&two_stop,serial_dev);
   client_fd = init_comm(serial_dev,baud,two_stop);
#else

    if ((client_fd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
        printf("\n Socket creation error \n");
        return -1;
    }
  
    setsockopt(client_fd, SOL_SOCKET, SO_REUSEADDR, &option, sizeof(option));
    // Didn't help with simh.
    //setsockopt(client_fd, SOL_SOCKET, TCP_NODELAY, &option, sizeof(option));

    serv_addr.sin_family = AF_INET;
    serv_addr.sin_port = htons(PORT);
  
    // Convert IPv4 and IPv6 addresses from text to binary
    // form
    if (inet_pton(AF_INET, "127.0.0.1", &serv_addr.sin_addr)
        <= 0) {
        printf(
            "\nInvalid address/ Address not supported \n");
        return -1;
    }
  
    if ((status
         = connect(client_fd, (struct sockaddr*)&serv_addr,
                   sizeof(serv_addr)))
        < 0) {
        printf("\nConnection Failed \n");
        return -1;
    }
#endif

   if (argc < 2) { 
      printf("Specify filename\n");
      exit(1);
   } 
   fd = open(argv[1], O_RDONLY);
   if (fd == 0) {
      printf("open failed\n");
      exit(1);
   }

   while (!terminate) {
      int rc;
      uint32_t word32;
      uint16_t word;
      rc = read(fd, &word32, sizeof(word32));
      if (rc != sizeof(word32)) {
         if (rc == 0) {
            printf("End of file\n");
            word32 = 0xffffffff;
         } else {
            printf("Bad read %d\n",rc);
            exit(1);
         }
      }
      if (word32 == 0) {
         printf("Sending filemark file %d last file %d records\n", file++, records);
         records = 0;
         send_word(1);
         word = get_word(); 
         if (word != 0) {
            printf("Wrong filemark echo\n");
            exit(1);
         }
      } else if (word32 == 0xffffffff) {
         printf("Sending done\n");
         send_word(0);
         word = get_word(); 
         printf("%d write errors\n", word);
      } else {
         uint8_t buf[4096];
         int checksum = 0;

         //printf("Sending record %d\n", word32);
         if (word32 > 4096) {
            printf("Bad record length %d\n",word32);
            exit(1);
         }
         rc = read(fd, buf, word32);
         if (rc != word32) {
            printf("Bad record read %d\n", rc);
            exit(1);
         }
         checksum = word32/2;
         send_word(2);
         send_word(word32/2);
         for (int i = 0; i < word32/2; i++) {
            uint16_t word2 = ((buf[i*2] & 077) << 6) | (buf[i*2+1] &077);
            //printf("Word %o\n",word2);
            send_word(word2);
            checksum += word2;
         }
	 word = get_word();
         checksum &= 07777;
         if (word != checksum) {
            printf("Bad checksum %o %o\n", word, checksum);
            exit(1);
         }
         // Discard trailing length
         rc = read(fd, &word32, sizeof(word32));
         records++;
      }
   }
}
